/*
 * motor.c
 *
 * Driver/Librería para el "Serial Motor Driver" ROB-09571 de Sparkfun
 *
 *  Created on: 01/12/2012
 *      Author: jaivar
 */

/* Librerias implicadas en este driver */

#include "LPC17xx.h"
#include "FreeRTOS.h"
#include "semphr.h"
#include "type.h"
#include "uart.h"
#include "printf.h"
#include "wifly.h"
#include "cpdsurveyor.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

extern volatile uint32_t UART1Count;
extern volatile uint8_t UART1Buffer[BUFSIZE];

char buffRx[BUFSIZE]; // buffer para recibir respuesta del Driver a través del UART
char buffTx[BUFSIZE]; // buffer para enviar comando al driver a través del UART
char buffLog[BUFSIZE]; // buffer para imprimir por el log la respuesta del UART

uint32_t sizeAsk; // int para alojar la cantidad de carácteres recibidos por respuesta en cada comando enviado al driver.

/* Declaramos una variable de tipo xSemaphoreHandle.  Esta será usada para
 * asegurarnos una proteccion de exclusion mutua evitando así enviar dos o más
 * comandos al driver simultaneamente a traves del UART definido. */

xSemaphoreHandle xMutexMotor;

/*
 *
 * La fución "MotorCleanBuffers" tiene como objetivo limpiar (vaciar) los bufers implicados en este driver.
 * y declarados más arriba.
 *
* Parámetros de entrada:
 *
 * Ninguno.
 *
 * Parámetros de salida:
 *
 * Ninguno.
 *
 */

void MotorCleanBuffers (){

	 memset(buffRx, 0x00, BUFSIZE);
     memset(buffTx, 0x00, BUFSIZE);
     memset(buffLog, 0x00, BUFSIZE);
}

/*
 *
 * La fución "MotorDelayMs" tiene como objetivo añadir una espera activa allí
 * donde sea llamada. Tiene como parámetro de entrada los milisegundos que
 * queramos que dure la espera.
 *
 * Parámetros de entrada:
 *
 * Ms = número de milisegundos que deseamos que dure la espera activa.
 *
 * Parámetros de salida:
 *
 * Ninguno.
 *
 */

void MotorDelayMs(int Ms)
{
	int i;
	// 7140 iteracciones son 1ms
	for(i=0; i<( ((uint32_t)7140)*Ms); i++){
	;
	}
}

/*
 *
 * La fución "MotorUART1scanf" tiene como objetivo recibir las respuestas trás el
 * envío de un comando al driver del motor a través del UART1.
 *
 * Parámetros de entrada:
 *
 * String = buffer donde guardaremos los carácteres recibidos por el UART1Buffer.
 * Timeout = Tiempo que dejaremos pasar para esuchar la respuesta.
 *
 * Parámetros de salida:
 *
 * NewCount = número de caracteres recibidos.
 *
 */

uint32_t MotorUART1scanf (char *String, uint32_t Timeout)
{
	int OldCount=0;
	int NewCount=0;

	UART1Count = 0; // clean buff
	LPC_UART1->IER = IER_THRE | IER_RLS | IER_RBR; // enable interrupt RX UART0
	MotorDelayMs(Timeout); // wait sometime for receive some commands
	do{
		OldCount = NewCount;
		MotorDelayMs(0);
		NewCount = UART1Count;
	}while(OldCount != NewCount); // wait a little time while we have new chars
	LPC_UART1->IER = IER_THRE | IER_RLS; // disable interrupt RX UART0

	strncpy(String, (char *)UART1Buffer, NewCount);
	String[NewCount] = '\0'; // add end string. Now we have a string

	return NewCount;
}

/*
 *
 * La fución "MotorLogResponse" tiene como objetivo ejecutar la función de lectura
 * de repuestas trás el envío de comandos e imprimirlas posteriormente por el UART
 * que utilicemos de log.
 *
 * Parámetros de entrada:
 *
 * TiempoEspera = Tiempo que escuchamos la respuesta.
 * UartLog = nº de Uart utilizado para imprimir la respuesta.
 *
 * Parámetros de salida:
 *
 * 0 = si se ha ejecutado correctamente.
 *
 */

uint32_t MotorLogResponse (int TiempoEspera, int UartLog){

	sizeAsk=MotorUART1scanf(buffRx,TiempoEspera);
	sprintf(buffLog,"******** LOG Respuesta Motor ******** : Hemos recibido %d chars con: %s \n", sizeAsk, buffRx);
	PrintText(UartLog,buffLog);

	return 0;
}

/*
 *
 * La fución "MotorInit" tiene como objetivo inicializar el driver del motor
 *  en el sistema, por lo que será de uso obligatorio para el correcto funcionamiento
 *  y deberá ser llamada antes de crear las tasks. También inicializará el semáforo
 *  anteriormente citado.
 *
 * Parámetros de entrada:
 *
 * UartPort = número entero con el nº de UART donde esta conectado el driver.
 * BaudRate = baudrate para poder negociar con el driver.
 *
 * Parámetros de salida:
 *
 * 0 = si se ha ejecutado correctamente.
 *
 */

uint32_t MotorInit (int UartPort, int BaudRate){

		// Pasamos como parametros el uart id & baud rate setting del UART
	    UARTInit(UartPort, BaudRate);

	    // Aprovechamos esta función para crear el semáforo antes de utilizarlo.
	    xMutexMotor = xSemaphoreCreateMutex();

	    return 0;
}

/*
 * La funcion "MotorMakeComand" tiene como objetivo formar un comando
 * con los parametros necesarios para el funcionamiento de cada motor.
 *
 * Los parámetros de entrada son:
 *
 * Motor = 1 o 2
 * Direccion = f (forward, hacia delante) o r (reverse, hacia atrás)
 * Velocidad = 0 (parado), 1 (11%), 2 (22%), 3 (33%)....9 (100% máxima velocidad)
 *
 * Los parámetros de salida son:
 *
 * BuffTx =
 */

uint32_t MotorMakeComand(int Motor, char *Direccion, int Velocidad){

	sprintf(buffTx,"%d%s%d\r",Motor,Direccion,Velocidad);

	//PrintText(3,buffTx); //log para ver si lo formamos correctamente.

	return 0;
}

/*
 * La fución "MotorSendComand" tiene como objetivo enviar el comando formado por
 * la función MotorMakeComand al Uart donde tenemos conectado el driver del motor.
 * Tiene integrada un semáforo para envitar atacar al Uart a la vez con más de un
 * comando.
 *
 * Parámetros de entrada:
 *
 * UartPort = número entero con el nº de UART donde esta conectado el driver.
 * BaudRate = baudrate para poder negociar con el driver.
 *
 * Parámetros de salida:
 *
 * 0 = si se ha ejecutado correctamente.
 *
 */

uint32_t MotorSendComand (int UartPort, char *Comando){

	xSemaphoreTake( xMutexMotor, portMAX_DELAY );
			{
	UARTSend(UartPort,(uint8_t *)Comando,strlen(Comando));
	MotorCleanBuffers();
			}
	xSemaphoreGive( xMutexMotor );

	return 0;
}

/*
 *
 * La fución "MotorControl"  tiene como objetivo mover el motor hacia adelante, atrás, derecha
 * izquierda, así como pararlo. Recibé las ordenes generadas desde la interface de usuario:
 * http://www.onesistemas.com/CPDSurveyor/
 *
 * Parámetros de entrada:
 *
 * Control = 0 (paramos motor), 1 (movemos hacia adelante), 2 (movemos hacia la izquierda),
 * 			 3 (movemos hacia la derecha), , 4 (movemos hacia atrás).
 *
 * Parámetros de salida:
 *
 * 0 = si se ha ejecutado correctamente.
 *
 */

uint32_t MotorControl (int Control){



	if (Control==0){
		//Parar el motor
		PrintText(3,"Parar\n\n");
		//formamos el comando parar. Seleccionamos motor trasero (tracción) y le damos velocidad 0
		MotorMakeComand(1,"f",0);
		//enviamos el comando al driver
		MotorSendComand(1,buffTx);
		//recibimos log con respuesta del driver del motor
		MotorLogResponse(1000,3);



	} else if (Control==1){
		//Control hacia adelante
		PrintText(3,"Adelante\n\n");
		//formamos el comando "adelante". Seleccionamos motor trasero (tracción), movemos tracción hacia el frente "f" y le damos máxima potencia "9"
		MotorMakeComand(1,"f",9);
		//enviamos el comando al driver
		MotorSendComand(1,buffTx);
		//recibimos log con respuesta del driver del motor
		MotorLogResponse(1000,3);
		//mantenemos el motor encendido y con dirección hacia delante durante 1000 Ms.
		MotorDelayMs(1000);
		//paramos motor
		MotorControl(0);

	} else if (Control==2){
		//giramos las ruedas hacia la derecha con el motor 2 (direccion)
		PrintText(3,"Giro de ruedas derecha\n\n");
		//formamos el comando "derecha". Seleccionamos motor delantero (direccion), seleccionamos hacia el frente "f" y le damos máxima potencia "9"
		MotorMakeComand(2,"f",9);
		//enviamos el comando al driver
		MotorSendComand(1,buffTx);
		//recibimos log con respuesta del driver del motor
		MotorLogResponse(1000,3);
		//mantenemos las ruedas giradas hacia la derecha durante 1000 Ms.
		MotorDelayMs(1000);

		//avanzamos adelante un poco para que realice el giro.
		MotorControl(1);

		//Volvemos a poner las ruedas rectas
		PrintText(3,"Ruedas rectas\n\n");
		MotorMakeComand(2,"f",0);
		MotorSendComand(1,buffTx);
		MotorLogResponse(1000,3);

	} else if (Control==3){
		//giramos las ruedas hacia la izquierda con el motor 2 (direccion)
		PrintText(3,"Giro de ruedas izquierda\n\n");
		//formamos el comando "izquierda". Seleccionamos motor delantero (direccion), seleccionamos hacia el reves "r" y le damos máxima potencia "9"
		MotorMakeComand(2,"r",9);
		//enviamos el comando al driver
		MotorSendComand(1,buffTx);
		//recibimos log con respuesta del driver del motor
		MotorLogResponse(1000,3);
		//mantenemos las ruedas giradas hacia la izquierda durante 1000 Ms.
		MotorDelayMs(1000);

		//avanzamos adelante un poco para que realice el giro.
		MotorControl(1);

		//Volvemos a poner las ruedas rectas
		PrintText(3,"Ruedas rectas\n\n");
		MotorMakeComand(2,"r",0);
		MotorSendComand(1,buffTx);
		MotorLogResponse(1000,3);

	} else if (Control == 4){
		//Control hacia atrás
		PrintText(3,"Atras \n\n");
		//formamos el comando "atrás". Seleccionamos motor trasero (tracción), movemos tracción hacia el revés "r" y le damos máxima potencia "9"
		MotorMakeComand(1,"r",9);
		//enviamos el comando al driver
		MotorSendComand(1,buffTx);
		//recibimos log con respuesta del driver del motor
		MotorLogResponse(1000,3);
		//mantenemos el motor encendido y con dirección hacia delante durante 1000 Ms.
		MotorDelayMs(1000);
		//paramos motor
		MotorControl(0);
	}

	return 0;
}

/*
 *
 * La fución "MotorCobertura"  tiene como objetivo encontrar un punto donde el rssi sea de mejor
 * calidad que el rssi límite fijado como calidad aceptable, de tal forma que encontrará automáticamente
 * una zona con rssi aceptable en caso de sobrepasar este límite.
 *
 * Parámetros de entrada:
 *
 * Rssi = rssi actual del dispositivo.
 * LimitRssi = rssi que se ha fijado como límite para ser aceptable.
 *
 * Parámetros de salida:
 *
 * 0 = si se ha ejecutado correctamente.
 *
 */

uint32_t MotorCobertura(int Rssi, int LimitRssi){

	//variable donde ireos guardando el último rssi obtenido
	int NewRssi;

	//como sabemos que estamos por debajo del rssi limite, moveremos el prototipo hacia delante para saber en que dirección mejora
	MotorControl(1);

	//buscaremos el rssi hasta que consigamos estar igual o por debajo del aceptable
	while (Rssi <= LimitRssi){

	//obtenemos el rssi actual, si al haberlo movido hacia delante mejora, seguirá hacia delante, si no, hacia detrás hasta conseguir el rssi aceptable.
	NewRssi=WiflyRssi();

		if (Rssi <= NewRssi){

				//sigue hacia delante hasta conseguir el rssi dentro de cobertura
				while (NewRssi <= LimitRssi){
     			//motor hacia delante
				MotorControl(1);
				NewRssi=WiflyRssi();
				}

			} else {

				//sigue hacia delante hasta conseguir el rssi dentro de cobertura
				while (NewRssi <= LimitRssi){
				//motor hacia atrás
				MotorControl(4);
				NewRssi=WiflyRssi();
				}
			}

	Rssi=NewRssi;

	}
	//enviamos el rssi una se está dentro de cobertura
	PrintText(3,"Fin control cobertua");
	CPDSurveyorSendRssi(Rssi);

	return 0;
}
