/*
 * wifly.c
 *
 * Driver/Librería para el dispositivo WiFly con FreeRtos
 *
 *  Created on: 01/11/2012
 *      Author: jaivar
 */

/* Librerias implicadas en este driver */

#include "LPC17xx.h"
#include "FreeRTOS.h"
#include "semphr.h"
#include "type.h"
#include "uart.h"
#include "printf.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>


#define RST_WIFLY_PIN       ( 22 )

extern volatile uint32_t UART3Count;
extern volatile uint8_t UART3Buffer[BUFSIZE];
extern volatile uint32_t UART0Count;
extern volatile uint8_t UART0Buffer[BUFSIZE];

char buffRx[BUFSIZE]; // buffer para recibir respuesta del Driver a través del UART
char buffTx[BUFSIZE]; // buffer para enviar comando al driver a través del UART
char buffLog[BUFSIZE]; // buffer para imprimir por el log la respuesta del UART

char str_ln[3] = {0x0D, 0x0A, 0x00}; // constante de tipo char para agregar un salto de línea en hexadecimal.

uint32_t sizeAsk; // int para alojar la cantidad de carácteres recibidos por respuesta en cada comando enviado al driver.

/* Declaramos una variable de tipo xSemaphoreHandle. Esta se usará para el mutex
 * del semáfomo que nos asegurará una protección de exclusión mutua para enviar
 * comando al Wifly por el Uart seleccionado.
 */

xSemaphoreHandle xMutexWifly;

/*
 *
 * La fución "WiflyCleanBuffers" tiene como objetivo limpiar (vaciar) los bufers implicados en este driver.
 * y declarados más arriba.
 *
* Parámetros de entrada:
 *
 * Ninguno.
 *
 * Parámetros de salida:
 *
 * Ninguno.
 *
 */

void WiflyCleanBuffers (){

	 memset(buffRx, 0x00, BUFSIZE);
     memset(buffTx, 0x00, BUFSIZE);
     memset(buffLog, 0x00, BUFSIZE);
}

/*
* La fución "WiflyDelayMs" tiene como objetivo añadir una espera activa allí
* donde sea llamada. Tiene como parámetro de entrada los milisegundos que
* queramos que dure la espera.
*
* Parámetros de entrada:
*
* Ms = número de milisegundos que deseamos que dure la espera activa.
*
* Parámetros de salida:
*
* Ninguno.
*
*/


void WiflyDelayMs(int ms)
{
	int i;
	// 7140 iteraciones son 1ms
	for(i=0; i<( ((uint32_t)7140)*ms); i++){
	;
	}
}

/*
* La fución "WiflyReset" tiene como objetivo realizar un reset al dispositivo
* Wifly a través del pin 24 de nuestra LPC.
*
* Parámetros de entrada:
*
* Ninguno.
*
* Parámetros de salida:
*
* Ninguno.
*
*/

void WiflyReset()
{

	// Initialize pin
	LPC_PINCON->PINSEL1 &= ( ~( 3 << 12 ) );
	LPC_GPIO0->FIODIR |= ( 1 << RST_WIFLY_PIN );
	// it made reset at Wifly
	LPC_GPIO0->FIOCLR = 1 << RST_WIFLY_PIN;
	WiflyDelayMs(1); // min 160us
	LPC_GPIO0->FIOSET = 1 << RST_WIFLY_PIN;

}

/*
 *
 * La fución "WiflyUART0scanf" tiene como objetivo recibir las respuestas trás el
 * envío de un comando al Wifly a través del UART1.
 *
 * Parámetros de entrada:
 *
 * String = buffer donde guardaremos los caracteres recibidos por el UART0Buffer.
 * Timeout = Tiempo que dejaremos pasar para esuchar la respuesta.
 *
 * Parámetros de salida:
 *
 * NewCount = número de caracteres recibidos.
 *
 */

uint32_t WiflyUART0scanf (char *String, uint32_t Timeout)
{
	int oldCount=0;
	int newCount=0;

	UART0Count = 0; // clean buff
	LPC_UART0->IER = IER_THRE | IER_RLS | IER_RBR; // enable interrupt RX UART0
	WiflyDelayMs(Timeout); // wait sometime for receive some commands
	do{
		oldCount = newCount;
		WiflyDelayMs(0);
		newCount = UART0Count;
	}while(oldCount != newCount); // wait a little time while we have new chars
	LPC_UART0->IER = IER_THRE | IER_RLS; // disable interrupt RX UART0

	strncpy(String, (char *)UART0Buffer, newCount);
	String[newCount] = '\0'; // add end string. Now we have a string

	return newCount;
}

/*
 *
 * La fución "WiflyLogResponse" tiene como objetivo ejecutar la función de lectura
 * de repuestas trás el envío de comandos e imprimirlas posteriormente por el UART
 * que utilicemos de log.
 *
 * Parámetros de entrada:
 *
 * TiempoEspera = Tiempo que escuchamos la respuesta.
 * UartLog = nº de Uart utilizado para imprimir la respuesta.
 *
 * Parámetros de salida:
 *
 * 0 = si se ha ejecutado correctamente.
 *
 */

uint32_t WiflyLogResponse (int TiempoEspera, int UartLog){

	sizeAsk=WiflyUART0scanf(buffRx,TiempoEspera);
	sprintf(buffLog,"******** LOG Respuesta WiFly ******** : Hemos recibido %d chars con: %s \n", sizeAsk, buffRx);
	PrintText(UartLog,buffLog);

	return 0;
}

/*
 *
 * La fución "WiflyInit" tiene como objetivo inicializar el dispositivo Wifly
 * en el sistema, por lo que será de uso obligatorio para el correcto funcionamiento
 * y deberá ser llamada antes de crear las tasks.
 *
 * Parámetros de entrada:
 *
 * UartPort = número entero con el nº de UART donde esta conectado el driver.
 * BaudRate = baudrate para poder negociar con el driver.
 *
 * Parámetros de salida:
 *
 * 0 = si se ha ejecutado correctamente.
 *
 */

uint32_t WiflyInit(int UartPort, int BaudRate){

	// En esta función crearemos el semáforo anteriormente citado.
    xMutexWifly = xSemaphoreCreateMutex();

    // Inicializamos el Uart donde esta conectado el dispositivo Wifly
    UARTInit(0, 9600);  /* uart id & baud rate setting WIFLY */

    //esperamos 3 seg. hasta que estén todos los dispositivos encendidos
    WiflyDelayMs(3000);
    PrintText(3,"********** ARRANQUE DEL SISTEMA **********\r\n");
    PrintText(3,"Realizamos reset al Wifly: \r\n");
	WiflyCleanBuffers();

	// Realizamos el reset y obtenemos la respuesta para ver si se ha realizado correctamente.
	WiflyReset();
	WiflyUART0scanf(buffRx,3000);
	PrintText(3,buffRx);

	return 0;
}

/*
 *
 * La fución "WiflyConsola" tiene como objetivo enviar el comando $$$ para poder
 * entrar en modo de configuración en el dispositvo Wifly.
 * Haremos uso del semaforo creado anteriormente para no enviar simultáneamente comandos
 * al dispositivo.
 *
 * Parámetros de entrada:
 *
 * UartPort = número entero con el nº de UART donde esta conectado el driver.
 *
 * Parámetros de salida:
 *
 * 0 = si se ha ejecutado correctamente.
 *
 */

uint32_t WiflyConsola (int UartPort){


	xSemaphoreTake( xMutexWifly, portMAX_DELAY );
	{
	sprintf(buffTx,"$$$");
	UARTSend(UartPort,(uint8_t *)buffTx,strlen(buffTx));
	WiflyCleanBuffers();
	}
	xSemaphoreGive( xMutexWifly );

	return 0;
}

/*
 *
 * La fución "WiflyComand" tiene como objetivo enviar un comando al dispositivo.
 * Haremos uso del semaforo al igual que en la función anterior.
 *
 * Parámetros de entrada:
 *
 * UartPort = número entero con el nº de UART donde esta conectado el driver.
 * Comando = buffer con el comando que deseamos enviar a través del UART.
 *
 * Parámetros de salida:
 *
 * 0 = si se ha ejecutado correctamente.
 *
 */

uint32_t WiflyComand (int UartPort, char *Comando){

	xSemaphoreTake( xMutexWifly, portMAX_DELAY );
	{
	strncat(Comando, str_ln, 3);
	UARTSend(UartPort,(uint8_t *)Comando,strlen(Comando));
	WiflyCleanBuffers();
	}
	xSemaphoreGive( xMutexWifly );

	return 0;
}

/*
 *
 * La fución "WiflyConect" tiene como objetivo configurar el wifly con el bssid
 * y password de la red Wify a la que nos deseamos conectar, además de conectarnos
 * a la misma.
 *
 * Parámetros de entrada:
 *
 * Password = cadena de caracteres alfanuméricos con la password de la red.
 * Red = bssid de la red Wify a la cual nos conectaremos.
 *
 * Parámetros de salida:
 *
 * 0 = si se ha ejecutado correctamente.
 *
 */


uint32_t WiflyConect (char *Password, char *Red){

	//Entramos en modo comando:
	WiflyConsola(0);

	//Imprimimos la respuesta para tener un log
	WiflyLogResponse(1000,3);

	//Preparamos comando y enviamos al Wifly
	sprintf(buffTx,"set wlan phrase %s",Password);
	WiflyComand(0,buffTx);

    //Imprimimos la respuesta para tener un log
	WiflyLogResponse(1000,3);

	//Preparamos comando y enviamos al Wifly
	sprintf(buffTx,"join %s",Red);
	WiflyComand(0,buffTx);

	//Imprimimos la respuesta para tener un log
	WiflyLogResponse(1000,3);

	return 0;

}

/*
 *
 * La fución "WiflyPing" tiene como objetivo realizar un ping sobre una dirección IP
 * que pasaremos como parámetro, de tal forma que podremos confirmar si tenemos conectividad
 * en la red donde se ha conectado nuestro dispositivo Wifly.
 *
 * Parámetros de entrada:
 *
 * Ip = dirección IP donde deseamos hacer ping desde el Wifly.
 *
 * Parámetros de salida:
 *
 * 0 = si se ha ejecutado correctamente.
 *
 */

uint32_t WiflyPing (char *Ip){

	PrintText(3,"\r\n********** Hacemos PING **********\r\n");

	//Preparamos comando y enviamos al Wifly
	sprintf(buffTx,"ping %s",Ip);
	WiflyComand(0,buffTx);

	//Imprimimos la respuesta para tener un log
	WiflyLogResponse(1000,3);

	return 0;

}

/*
 *
 * La fución "WiflyIp" tiene como objetivo mostrarnos la ip asignada trás conectarse
 * a una red.
 *
 * Parámetros de entrada:
 *
 * Ninguno.
 *
 * Parámetros de salida:
 *
 * 0 = si se ha ejecutado correctamente.
 *
 */

uint32_t WiflyIp(){

	//Variables de tipo char para realizar el Parser
	char PrefixIp[2]="IP";
	char BuffParser[BUFSIZE];

	PrintText(3,"\r\n********** IP asignada **********\r\n");

	//Preparamos el comando y lo enviamos al Wifly
	sprintf(buffTx,"get ip");
	WiflyComand(0,buffTx);

	//Hacemos el Parser e imprimimos la respuesta para tener un log
	sizeAsk=WiflyUART0scanf(buffRx,1000);
	sprintf(BuffParser,"%s", strstr(buffRx,PrefixIp)+3);
	strtok(BuffParser,":");

	sprintf(buffLog,"La direccion asignada es: %s \n",BuffParser);
	PrintText(3,buffLog);

	return 0;
}

/*
 *
 * La fución "WiflyCanal" tiene como objetivo mostrarnos el canal asignado trás conectarse
 * a una red.
 *
 * Parámetros de entrada:
 *
 * Ninguno.
 *
 * Parámetros de salida:
 *
 * 0 = si se ha ejecutado correctamente.
 *
 */

uint32_t WiflyCanal(){

	//Variables de tipo char para realizar el Parser
	char PrefixCanal[3]="han";
	char BuffParser[BUFSIZE];

	PrintText(3,"\r\n********** Canal asignado **********\r\n");

	//Preparamos el comando y lo enviamos al Wifly
	sprintf(buffTx,"get wlan");
	WiflyComand(0,buffTx);

	//Hacemos el Parser e imprimimos la respuesta para tener un log
	sizeAsk=WiflyUART0scanf(buffRx,1000);
	sprintf(BuffParser,"%s", strstr(buffRx,PrefixCanal)+4);
	strtok(BuffParser,"\r\n");
	sprintf(buffLog,"El canal asignado es: %s \n",BuffParser);
	PrintText(3,buffLog);

	return 0;
}

/*
 *
 * La fución "WiflyRssi" tiene como objetivo devolvernos el rssi que tenemos en el momento
 * de ejecutar esta función.
 *
 * Parámetros de entrada:
 *
 * Ninguno.
 *
 * Parámetros de salida:
 *
 * rssi = valor tipo int con el rssi obtenido.
 *
 */

uint32_t WiflyRssi(){

	int rssi;

	//Variables de tipo char para realizar el Parser
	char PrefixRssi[2]="RS";
	char BuffParser[BUFSIZE];

	PrintText(3,"\r\n********** RSSI **********\r\n");

	//Preparamos el comando y lo enviamos al Wifly
	sprintf(buffTx,"show rssi");
	WiflyComand(0,buffTx);

	//Hacemos el Parser e imprimimos la respuesta para tener un log
	sizeAsk=WiflyUART0scanf(buffRx,1000);
	sprintf(BuffParser,"%s", strpbrk(buffRx,PrefixRssi)+6);
	strtok(BuffParser,")");
	rssi = atoi(BuffParser);
	sprintf(buffLog,"El RSSI es: %s\n",BuffParser);
	PrintText(3,buffLog);
	WiflyCleanBuffers();

	return rssi;
}

/*
 *
 * La fución "WiflyTimePower" tiene como objetivo mostrarnos el tiempo que lleva encendido el
 * dispositivo Wifly.
 *
 * Parámetros de entrada:
 *
 * Ninguno.
 *
 * Parámetros de salida:
 *
 * 0 = si se ha ejecutado correctamente.
 *
 */

uint32_t WiflyTimePower(){

	//Variables de tipo char para realizar el Parser
	char Prefixtime[4]="U";
	char BuffParser[BUFSIZE];

	PrintText(3,"\r\n********** Tiempo de encendido Wifly **********\r\n");

	//Preparamos el comando y lo enviamos al Wifly
	sprintf(buffTx,"show time");
	WiflyComand(0,buffTx);

	//Hacemos el Parser e imprimimos la respuesta para tener un log
	sizeAsk=WiflyUART0scanf(buffRx,1000);
	sprintf(BuffParser,"%s", strpbrk(buffRx,Prefixtime)+7);
	strtok(BuffParser,"\r\n");
	sprintf(buffLog,"El tiempo es de: %s \n", BuffParser);
	PrintText(3,buffLog);

    return 0;
}

/*
 *
 * La fución "WiflyPowerTx" tiene como objetivo mostrarnos la potencia de transmisión
 * del dispositivo Wifly.
 *
 * Parámetros de entrada:
 *
 * Ninguno.
 *
 * Parámetros de salida:
 *
 * 0 = si se ha ejecutado correctamente.
 *
 */


uint32_t WiflyPowerTx(){

	//Variables de tipo char para realizar el Parser
	char PrefixPowerTx[4]="Tx";
	char BuffParser[BUFSIZE];

	PrintText(3,"\r\n********** Ver potencia Tx **********\r\n");

	//Preparamos el comando y lo enviamos al Wifly
	sprintf(buffTx,"get wlan");
	WiflyComand(0,buffTx);

	//Hacemos el Parser e imprimimos la respuesta para tener un log
	sizeAsk=WiflyUART0scanf(buffRx,1000);
	sprintf(BuffParser,"%s", strstr(buffRx,PrefixPowerTx)+8);
	strtok(BuffParser,"\r\n");
	sprintf(buffLog,"La potencia de Tx es: %s \n",BuffParser);
	PrintText(3,buffLog);

	return 0;
}

/*
 *
 * La fución "WiflyChangePowerTx" tiene como objetivo cambiar la potencia de transmisión por la que
 * indiquémos como parámetro de entrada.
 *
 * Parámetros de entrada:
 *
 * Potencia = Potencia que deseamos que transmita el dispositivo Wifly.
 *
 * Parámetros de salida:
 *
 * 0 = si se ha ejecutado correctamente.
 *
 */

uint32_t WiflyChangePowerTx(uint32_t Potencia){

	PrintText(3,"\r\n********** Cambiamos la potencia Tx **********\r\n");

	//Preparamos comando y enviamos al Wifly
	sprintf(buffTx,"set wlan tx %d",Potencia);
	WiflyComand(0,buffTx);

	//Comprobamos si se ha realizado el cambio
	PrintText(3,"\r\n********** La nueva potencia Tx es: **********\r\n");
	WiflyPowerTx();

	return 0;
}

